\\ Copyright 2013, 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

read("memoize.gp");

atan2(y,x) = my(angle=arg(x+I*y)); if(angle<0,angle+2*Pi,angle);
deg(x) = x/180*Pi;

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_equal(got,want) =
{
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok"));
}

\\-------------

Iz(k)  = 3/20 * (5^k - 1);

b=1+2*I;
r5dragon_midpoints(k) = {
  my(end=1,v=[1/2]);
  for(i=1,k,
     v = concat(v,
         concat(apply(z->end       + z*I, v),
         concat(apply(z->end*(1+I) - z,   v),
         concat(apply(z->end*I     + z*I, v),
                apply(z->end*2*I   + z,   v)))));
     end *= b);
  v;
}
check_equal(r5dragon_midpoints(0), [1/2]);
check_equal(r5dragon_midpoints(1), [1/2, 1+1/2*I, 1/2+I, 3/2*I, 1/2+2*I]);

r5dragon_end(k) = b^k;
check_equal(r5dragon_end(0), 1);
check_equal(r5dragon_end(1), 1+2*I);

r5dragon_centroid(k) = r5dragon_end(k)/2;

Iz_by_sum(k) =
{
  my(v = r5dragon_midpoints(k));
  my(centroid = r5dragon_centroid(k));
  v = apply(z->(z-centroid), v);
  sum(i=1,#v, v[i]^2) / #v;
}
Iz_by_sum = memoize(Iz_by_sum);

Ika_by_sum(k,a) =
{
  my(v = r5dragon_midpoints(k));
  my(centroid = r5dragon_centroid(k));
  my(rotate = 1/((b/abs(b))^k * exp(a*I)));
  v = apply(z->(z-centroid)*rotate, v);
  sum(i=1,#v, imag(v[i])^2) / #v;
}
Ika_by_sum = memoize(Ika_by_sum);
check_equal(Ika_by_sum(0,0), 0);
check_equal(Ika_by_sum(0,Pi/2), 0);

\\-------------

\\ cos(a+atan(2/1)) = cos(a)*1/sqrt(5) - sin(a)*2/sqrt(5)
\\ cos(a+atan(2/1))^2 = 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2

\\ sin(a+atan(2/1)) = cos(a)*2/sqrt(5) + sin(a)*1/sqrt(5)
\\ sin(a+atan(2/1))^2 = 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2

\\ cos(a+atan(2/1))*sin(a+atan(2/1))
\\  = 2/5*cos(a)^2 - 3/5*cos(a)*sin(a) - 2/5*sin(a)^2

check_equal(cos(atan(2/1)), 1/sqrt(5))
check_equal(vector(256,k,my(a=2*Pi/k); cos(a+atan(2/1))), \
            vector(256,k,my(a=2*Pi/k); cos(a)*1/sqrt(5) - sin(a)*2/sqrt(5)));
check_equal(vector(256,k,my(a=2*Pi/k); cos(a+atan(2/1))^2), \
            vector(256,k,my(a=2*Pi/k); 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2));

check_equal(vector(256,k,my(a=2*Pi/k); cos(a+atan(2/1))*sin(a+atan(2/1))), \
            vector(256,k,my(a=2*Pi/k); 2/5*cos(a)^2 - 3/5*cos(a)*sin(a) - 2/5*sin(a)^2 ));

check_equal(vector(256,k,my(a=2*Pi/k); sin(a+atan(2/1))), \
            vector(256,k,my(a=2*Pi/k); cos(a)*2/sqrt(5) + sin(a)*1/sqrt(5)));
check_equal(vector(256,k,my(a=2*Pi/k); sin(a+atan(2/1))^2), \
            vector(256,k,my(a=2*Pi/k); 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2 ));

\\ 1st,5th add 2/1
\\ Ix*cos(a)^2 + Ixy*cos*sin + Iy*sin^2
\\ = Ix  * ( 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2 )
\\ + Ixy * ( 2/5*cos(a)^2 - 3/5*cos(a)*sin(a)  - 2/5*sin(a)^2 )
\\ + Iy  * ( 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2 )


\\-------------

\\ cos(a-atan(1/2)) = cos(a)*2/sqrt(5) + sin(a)*1/sqrt(5)
\\ cos(a-atan(1/2))^2 = 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2

\\ sin(a-atan(1/2)) = - cos(a)*1/sqrt(5) + sin(a)*2/sqrt(5)
\\ sin(a-atan(1/2))^2 = 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2

\\ cos(a-atan(1/2))*sin(a-atan(1/2))
\\  = -2/5*cos(a)^2 + 3/5*cos(a)*sin(a) + 2/5*sin(a)^2

check_equal(vector(256,k,my(a=2*Pi/k); cos(a-atan(1/2)) ), \
            vector(256,k,my(a=2*Pi/k); cos(a)*2/sqrt(5) + sin(a)*1/sqrt(5) ));
check_equal(vector(256,k,my(a=2*Pi/k); cos(a-atan(1/2))^2 ), \
            vector(256,k,my(a=2*Pi/k); 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2 ));

check_equal(vector(256,k,my(a=2*Pi/k); cos(a-atan(1/2))*sin(a-atan(1/2)) ), \
            vector(256,k,my(a=2*Pi/k); -2/5*cos(a)^2 + 3/5*cos(a)*sin(a) + 2/5*sin(a)^2 ));

check_equal(vector(256,k,my(a=2*Pi/k); sin(a-atan(1/2)) ), \
            vector(256,k,my(a=2*Pi/k); - cos(a)*1/sqrt(5) + sin(a)*2/sqrt(5) ));
check_equal(vector(256,k,my(a=2*Pi/k); sin(a-atan(1/2))^2 ), \
            vector(256,k,my(a=2*Pi/k); 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2 ));

\\ 2nd,4th sub 1/2
\\ Ix*cos(a)^2 + Ixy*cos*sin + Iy*sin^2
\\ = Ix  * ( 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2 )
\\ + Ixy * (-2/5*cos(a)^2 + 3/5*cos(a)*sin(a)  + 2/5*sin(a)^2 )
\\ + Iy  * ( 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2 )

\\-------------

\\ cos(a-atan(2/-1)) = cos(a)*1/sqrt(5) - sin(a)*2/sqrt(5)
\\ cos(a-atan(2/-1))^2 = 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2

\\ sin(a-atan(2/-1)) = cos(a)*2/sqrt(5) - sin(a)*-1/sqrt(5)
\\ sin(a-atan(2/-1))^2 = 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2

\\ cos(a-atan(2/-1))*sin(a-atan(2/-1))
\\  = 2/5*cos(a)^2 - 3/5*cos(a)*sin(a) + -2/5*sin(a)^2

check_equal(vector(256,k,my(a=2*Pi/k); cos(a-atan(2/-1)) ), \
            vector(256,k,my(a=2*Pi/k); cos(a)*1/sqrt(5) - sin(a)*2/sqrt(5) ));
check_equal(vector(256,k,my(a=2*Pi/k); cos(a-atan(2/-1))^2 ), \
            vector(256,k,my(a=2*Pi/k); 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2 ));

check_equal(vector(256,k,my(a=2*Pi/k); cos(a-atan(2/-1))*sin(a-atan(2/-1)) ), \
            vector(256,k,my(a=2*Pi/k); 2/5*cos(a)^2 - 3/5*cos(a)*sin(a) + -2/5*sin(a)^2 ));

check_equal(vector(256,k,my(a=2*Pi/k); sin(a-atan(2/-1)) ), \
            vector(256,k,my(a=2*Pi/k); cos(a)*2/sqrt(5) - sin(a)*-1/sqrt(5) ));
check_equal(vector(256,k,my(a=2*Pi/k); sin(a-atan(2/-1))^2 ), \
            vector(256,k,my(a=2*Pi/k); 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2 ));

\\ 3rd sub 2/-1
\\ Ix*cos(a)^2 + Ixy*cos*sin + Iy*sin^2
\\ = Ix  * ( 1/5*cos(a)^2 - 4/5*cos(a)*sin(a) + 4/5*sin(a)^2 )
\\ + Ixy * ( 2/5*cos(a)^2 - 3/5*cos(a)*sin(a) - 2/5*sin(a)^2 )
\\ + Iy  * ( 4/5*cos(a)^2 + 4/5*cos(a)*sin(a) + 1/5*sin(a)^2 )

check_equal(vector(256,k,my(a=2*Pi/k + Pi); cos(a-atan(2/-1))*sin(a-atan(2/-1)) ), \
            vector(256,k,my(a=2*Pi/k); 2/5*cos(a)^2 - 3/5*cos(a)*sin(a) + -2/5*sin(a)^2 ));
check_equal(vector(256,k,my(a=2*Pi/k+Pi); cos(a-atan(2/-1))*sin(a-atan(2/-1)) ), \
            vector(256,k,my(a=2*Pi/k); 2/5*cos(a)^2 - 3/5*cos(a)*sin(a) + -2/5*sin(a)^2 ));
check_equal(vector(256,k,my(a=2*Pi/k+Pi); sin(a-atan(2/-1))^2 ), \
            vector(256,k,my(a=2*Pi/k); 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2 ));


\\-------------

\\ r^2 sub 1/2 sub Pi/4
\\ 
\\ sin(a-atan(1/2)-Pi/4)   = cos(a)*-3/sqrt(10) + sin(a)*1/sqrt(10)
\\ sin(a-atan(1/2)-Pi/4)^2 = 9/10*cos(a)^2 - 3/5*cos(a)*sin(a)  + 1/10*sin(a)^2

check_equal(vector(256,k,my(a=2*Pi/k); sin(a-atan(1/2)-Pi/4) ), \
            vector(256,k,my(a=2*Pi/k); cos(a)*-3/sqrt(10) + sin(a)*1/sqrt(10) ));
check_equal(vector(256,k,my(a=2*Pi/k); sin(a-atan(1/2)-Pi/4)^2 ), \
            vector(256,k,my(a=2*Pi/k); 9/10*cos(a)^2 - 3/5*cos(a)*sin(a) + 1/10*sin(a)^2 ));

\\-------------
\\ t factor 2*1/5*5^(k-1) = 2/5*5^(k-1)
\\ r factor               = 2/5*1/sqrt(2)*5^(k-1)

\\-------------

Ika = Ika_by_sum;

Ika_by_parallel_axis(k,a) = {
  if(k==0,return(0));
    2/5* Ika(k-1, a + atan(2/1))
  + 2/5* Ika(k-1, a - atan(1/2))
  + 1/5* Ika(k-1, a - atan(2/-1))
  + 2*1/5 *(           sqrt(5)^(k-1)*sin(a - atan(1/2)) )^2         /* m*t^2 */
  + 2*1/5 *( 1/sqrt(2)*sqrt(5)^(k-1)*sin(a - atan(1/2) - Pi/4) )^2; /* m*r^2 */
}
check_equal(vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_sum(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_parallel_axis(k,a))))

\\ k=1
\\ a=1 \\-atan(2/1);
\\ Ika(k,a)
\\ Ika_by_parallel_axis(k,a)
\\ print("t ", 2*1/5*(           sqrt(5)^(k-1)*sin(a - atan(1/2)) )^2);
\\ print("r ", 2*1/5*( 1/sqrt(2)*sqrt(5)^(k-1)*sin(a - atan(1/2) - Pi/4) )^2);
\\ 
\\ v = r5dragon_midpoints(k);
\\ centroid = r5dragon_centroid(k);
\\ rotate = 1/((b/abs(b))^k * exp(a*I));
\\ v = apply(z->(z-centroid)*rotate, v);
\\ v = apply(z->imag(z)^2, v);
\\ v /= 5;
\\ print("by sum ", v);
\\ print("Ika by sum ", vecsum(v));
\\ print("Ika_by_sum ", Ika_by_sum(k,a));
\\ print("Ika_by_parallel_axis ", Ika_by_parallel_axis(k,a));
\\ print("t sin^2 ", sin(a - atan(1/2))^2);
\\ print("r sin^2 ", sin(a - atan(1/2) - Pi/4)^2);
\\ print("2/5 t sin^2 ", 2/5*sin(a - atan(1/2))^2);
\\ print("2/5 r (1/sqrt2 sin)^2 ", 2/5*(1/sqrt(2)*sin(a - atan(1/2) - Pi/4))^2);
\\ quit

\\-------------

\\ 1st,5th add 2/1
\\ Ix*cos(a)^2 + Ixy*cos*sin + Iy*sin^2
\\ = Ix  * ( 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2 )
\\ + Ixy * ( 2/5*cos(a)^2 - 3/5*cos(a)*sin(a)  - 2/5*sin(a)^2 )
\\ + Iy  * ( 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2 )

\\ 2nd,4th sub 1/2
\\ Ix*cos(a)^2 + Ixy*cos*sin + Iy*sin^2
\\ = Ix  * ( 4/5*cos(a)^2 + 4/5*cos(a)*sin(a)  + 1/5*sin(a)^2 )
\\ + Ixy * (-2/5*cos(a)^2 + 3/5*cos(a)*sin(a)  + 2/5*sin(a)^2 )
\\ + Iy  * ( 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2 )

\\ 3rd sub 2/-1
\\ Ix*cos(a)^2 + Ixy*cos*sin + Iy*sin^2
\\ = Ix  * ( 1/5*cos(a)^2 - 4/5*cos(a)*sin(a) + 4/5*sin(a)^2 )
\\ + Ixy * ( 2/5*cos(a)^2 - 3/5*cos(a)*sin(a) - 2/5*sin(a)^2 )
\\ + Iy  * ( 4/5*cos(a)^2 + 4/5*cos(a)*sin(a) + 1/5*sin(a)^2 )

\\ r
\\ check_equal(vector(256,k,my(a=2*Pi/k); sin(a-atan(1/2)-Pi/4)^2 ), \
\\             vector(256,k,my(a=2*Pi/k); 9/10*cos(a)^2 - 3/5*cos(a)*sin(a) + 1/10*sin(a)^2 ));

\\ t
\\ check_equal(vector(256,k,my(a=2*Pi/k); sin(a-atan(1/2))^2 ), \
\\             vector(256,k,my(a=2*Pi/k); 1/5*cos(a)^2 - 4/5*cos(a)*sin(a)  + 4/5*sin(a)^2 ));

{
Ix(k)  = if(k==0,return(0));
          2/5* 1/5*Ix(k-1) + 2/5*  2/5*Ixy(k-1) + 2/5* 4/5*Iy(k-1)  \\ 1,5
       +  2/5* 4/5*Ix(k-1) + 2/5* -2/5*Ixy(k-1) + 2/5* 1/5*Iy(k-1)  \\ 2,4
       +  1/5* 1/5*Ix(k-1) + 1/5*  2/5*Ixy(k-1) + 1/5* 4/5*Iy(k-1)  \\ 3
       + 9/10 * 2*1/5 * 1/2*5^(k-1)   \\ 2 * 1/5 * r^2
       + 1/5  * 2*1/5 * 5^(k-1)       \\ 2 * 1/5 * t^2
        ;
}
{
Ixy(k) = if(k==0,return(0));
         2/5* -4/5*Ix(k-1) + 2/5* -3/5*Ixy(k-1) + 2/5*  4/5*Iy(k-1)  \\ 1st,5th
       + 2/5*  4/5*Ix(k-1) + 2/5*  3/5*Ixy(k-1) + 2/5* -4/5*Iy(k-1)  \\ 2,4
       + 1/5* -4/5*Ix(k-1) + 1/5* -3/5*Ixy(k-1) + 1/5*  4/5*Iy(k-1)  \\ 3
       + -3/5 * 2*1/5 * 1/2*5^(k-1)   \\ r
       + -4/5 * 2*1/5 * 5^(k-1)       \\ t
       ;
}
{
Iy(k)  = if(k==0,return(0));
          2/5* 4/5*Ix(k-1) + 2/5* -2/5*Ixy(k-1) + 2/5* 1/5*Iy(k-1)  \\ 1st,5th
       +  2/5* 1/5*Ix(k-1) + 2/5*  2/5*Ixy(k-1) + 2/5* 4/5*Iy(k-1)  \\ 2,4
       +  1/5* 4/5*Ix(k-1) + 1/5* -2/5*Ixy(k-1) + 1/5* 1/5*Iy(k-1)  \\ 3
       + 1/10 * 2*1/5 * 1/2*5^(k-1)   \\ r
       + 4/5  * 2*1/5 * 5^(k-1)       \\ t
       ;
}
Ix = memoize(Ix);
Ixy= memoize(Ixy);
Iy = memoize(Iy);

Ika(k,a) = Ix(k)*cos(a)^2 + Ixy(k)*cos(a)*sin(a) + Iy(k)*sin(a)^2;

check_equal(vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika(k+1,a))), \
            vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_sum(k+1,a))))

\\----------------------
\\ mutual recurrences
{
Ix(k)  = if(k==0,return(0));
       11/25*Ix(k-1) + 2/25*Ixy(k-1) + 14/25*Iy(k-1)  +  13/50*5^(k-1);
}
{
Ixy(k) = if(k==0,return(0));
       -4/25*Ix(k-1) + -3/25*Ixy(k-1) + 4/25*Iy(k-1) - 11/25*5^(k-1);
}
{
Iy(k)  = if(k==0,return(0));
       14/25*Ix(k-1) + -2/25*Ixy(k-1) + 11/25*Iy(k-1) + 17/50*5^(k-1);
}
Ix = memoize(Ix);
Ixy= memoize(Ixy);
Iy = memoize(Iy);

check_equal(vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika(k+1,a))), \
            vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_sum(k+1,a))))



\\ Iy(k) - Ix(k)
\\ = 14/25*Ix(k-1) + -2/25*Ixy(k-1) + 11/25*Iy(k-1) + 17/50*5^(k-1)
\\ -(11/25*Ix(k-1) + 2/25*Ixy(k-1) + 14/25*Iy(k-1)  +  13/50*5^(k-1) )
\\ =  3/25*Ix(k-1) + -4/25*Ixy(k-1) + -3/25*Iy(k-1)  +  4/50*5^(k-1)
\\
\\ Ix(k) + Ix(k)
\\ = 11/25*Ix(k-1) + 2/25*Ixy(k-1) + 14/25*Iy(k-1)  +  13/50*5^(k-1)
\\ + 14/25*Ix(k-1) + -2/25*Ixy(k-1) + 11/25*Iy(k-1) + 17/50*5^(k-1)
\\ =       Ix(k-1)                  +       Iy(k-1)  + 30/50*5^(k-1)
\\
\\ is Iz(k) = Iz(k-1) + 3/5*5^(k-1)

\\-------------
{
Ix(k)  = if(k==0,return(0));
       11/25*Ix(k-1) + 2/25*Ixy(k-1) + 14/25*(3/20*(5^(k-1)-1) - Ix(k-1))  +  13/50*5^(k-1);
}
{
Ixy(k) = if(k==0,return(0));
       -4/25*Ix(k-1) + -3/25*Ixy(k-1) + 4/25*(3/20*(5^(k-1)-1) - Ix(k-1)) - 11/25*5^(k-1);
}
{
Iy(k)  = if(k==0,return(0));
       14/25*Ix(k-1) + -2/25*Ixy(k-1) + 11/25*(3/20*(5^(k-1)-1) - Ix(k-1)) + 17/50*5^(k-1);
}
Ix = memoize(Ix);
Ixy= memoize(Ixy);
Iy = memoize(Iy);

check_equal(vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika(k+1,a))), \
            vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_sum(k+1,a))))

\\-------------
{
Ix(k)  = if(k==0,return(0));
       -3/25*Ix(k-1) + 2/25*Ixy(k-1)  + 43/125*5^(k-1) - 21/250;  
}
{
Ixy(k) = if(k==0,return(0));
       -8/25*Ix(k-1) + -3/25*Ixy(k-1)  - 52/125*5^(k-1) - 3/125;
}
{
Iy(k)  = if(k==0,return(0));
        3/25*Ix(k-1) + -2/25*Ixy(k-1)  + 203/500*5^(k-1) - 33/500;
}
Ix = memoize(Ix);
Ixy= memoize(Ixy);
Iy = memoize(Iy);

check_equal(vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika(k+1,a))), \
            vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_sum(k+1,a))))

\\-------------
\\ Ixy(k-1) = 25/2*Ix(k) + 3/2*Ix(k-1) - 43/10*5^(k-1) + 21/20
\\
\\ 25/2*Ix(k+1) + 3/2*Ix(k) - 43/10*5^(k) + 21/20
\\ =
\\ - 8/25*Ix(k-1) + -3/25*(25/2*Ix(k) + 3/2*Ix(k-1) - 43/10*5^(k-1) + 21/20)  - 52/125*5^(k-1) - 3/125

check_equal(vector(5,k, 25/2*Ix(k+1) + 3/2*Ix(k) - 43/10*5^(k) + 21/20), \
            vector(5,k, - 8/25*Ix(k-1) + -3/25*(25/2*Ix(k) + 3/2*Ix(k-1) - 43/10*5^(k-1) + 21/20)  - 52/125*5^(k-1) - 3/125))

\\ 25/2*Ix(k+1) =
\\ - 3/2*Ix(k) + 43/10*5^(k) - 21/20
\\ - 8/25*Ix(k-1) - 3/25*25/2*Ix(k) - 3/25*3/2*Ix(k-1) + 3/25*43/10*5^(k-1)
\\ - 3/25*21/20  - 52/125*5^(k-1) - 3/125
\\
{
check_equal(vector(5,k, 25/2*Ix(k+1)), \
            vector(5,k,
- 3/2*Ix(k) + 43/10*5^(k) - 21/20
- 8/25*Ix(k-1) - 3/25*25/2*Ix(k) - 3/25*3/2*Ix(k-1) + 3/25*43/10*5^(k-1)
- 3/25*21/20  - 52/125*5^(k-1) - 3/125
))
}

\\ 25/2*Ix(k+1) = - 3*Ix(k) - 1/2*Ix(k-1) + 108/5* 5^(k-1) - 6/5

check_equal(vector(5,k, 25/2*Ix(k+1)), \
            vector(5,k, - 3*Ix(k) - 1/2*Ix(k-1) + 108/5*5^(k-1) - 6/5 ))

check_equal(vector(5,k, Ix(k+1)), \
            vector(5,k, - 6/25*Ix(k) - 1/25*Ix(k-1) + 216/125*5^(k-1) - 12/125 ))

{
Ix_by_rec(k)  = if(k==0,return(0));
         if(k==1,return(13/50));
         -6/25*Ix(k-1) - 1/25*Ix(k-2) + 216/125* 5^(k-2) - 12/125;
}
Ix_by_rec = memoize(Ix_by_rec);

check_equal(vector(100,k,my(k=k-1); Ix(k)), \
            vector(100,k,my(k=k-1); Ix_by_rec(k)))

Ix = Ix_by_rec;
check_equal(vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika(k+1,a))), \
            vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_sum(k+1,a))))

\\------------
\\ 0 + 13/50*x + 981/625*x^2 + ...

gFives(x) = 1/(1-5*x);
gOnes(x) = 1/(1-x);

\\ g(x) + 6/25*g(x)*x + 1/25*g(x)*x^2
\\ == 216/125 * x^2/(1-5*x) - 12/125*x^2/(1-x)
\\    + 13/50*x
\\
check_equal(981/625, \
            -6/25*13/50 - 1/25*0 + 216/125*1 - 12/125)

g(x) = 1/(1 + 6/25*x + 1/25*x^2) * ( 13/50*x + 216/125 * x^2/(1-5*x) - 12/125*x^2/(1-x) );
check_equal(Vec(g(x) - O(x^101)), \
            vector(100,k, Ix(k)))

\\ g(x) = 1/10*(13*x^3 + 18*x^2 + 65*x) \
\\        / ( (25 + 6*x + x^2) * (1-5*x) * (1-x));
check_equal(1/10*(13*x^3 + 18*x^2 + 65*x) \
            / ( (25 + 6*x + x^2) * (1-5*x) * (1-x)) \
            == g(x), 1)

\\ g(x) = (75/328 + 65/328*x) / (25 + 6*x + x^2) \
\\        + (27/410) / (1-5*x) \
\\        + (-3/40) * (1-x);
check_equal((75/328 + 65/328*x) / (25 + 6*x + x^2) \
            + 27/410 / (1-5*x) \
            - 3/40 / (1-x) \
            == g(x), 1)

check_equal( (x + 3+4*I)* (x + 3-4*I) == 25 + 6*x + x^2, 1)

\\ A/(x + 3+4*I) + B/(x + 3-4*I) == (75/328 + 65/328*x) / (25 + 6*x + x^2)
\\ A + B/(x + 3-4*I)*(x + 3+4*I) == (75/328 + 65/328*x)/(x + 3-4*I)
\\ at x=-(3+4*I)   A = 65/656 - 15/328*I
\\
\\ A*(x + 3-4*I)/(x + 3+4*I) + B == (75/328 + 65/328*x) / (x + 3+4*I)
\\ at x=-(3-4*I)   B = 65/656 + 15/328*I

\\ (65/656 - 15/328*I)*(x + 3-4*I) + (65/656 + 15/328*I)*(x + 3+4*I)

check_equal(  (65/656 - 15/328*I)/(3+4*I) / (1 - (-3/25 + 4/25*I)*x) \
            + (65/656 + 15/328*I)/(3-4*I) / (1 - (-3/25 - 4/25*I)*x) \
            + 27/410 / (1-5*x) \
            - 3/40 / (1-x) \
            == g(x), 1)

\\ Vec(1/(x + 3+4*I) - O(x^10))

\\-------------

\\ polroots(25*x^2 +6*x + 1)
check_equal(polroots(x^2 + 6/25*x + 1/25), \
            [ -3/25 - 4/25*I, \
              -3/25 + 4/25*I ]);
check_equal( abs( -3/25 + 4/25*I ), 1/5)

\\ A*(-3/25 + 4/25*I)^k + B*(-3/25 - 4/25*I)^k + C*5^k + D
check_equal(Ix_by_rec(0), 0)
check_equal(Ix_by_rec(1), 13/50)
check_equal(Ix_by_rec(2), 981/625)
check_equal(Ix_by_rec(3), 254903/31250)

{
check_equal(
Mat([ vector(4,k, (-3/25 + 4/25*I)^(k-1))~,
      vector(4,k, (-3/25 - 4/25*I)^(k-1))~,
      vector(4,k, 5^(k-1))~,
      vector(4,k, 1)~
    ])
^-1 \
* vector(4,k,Ix_by_rec(k-1))~
,
[3/656 - 7/328*I, 3/656 + 7/328*I, 27/410, -3/40]~
)
}

\\ [           1,                       1,              1, 1; \
\\        3/25 + 4/25*I,           3/25 - 4/25*I,       5, 1; \
\\      -7/625 + 24/625*I,       -7/625 - 24/625*I,    25, 1; \
\\  -117/15625 + 44/15625*I, -117/15625 - 44/15625*I, 125, 1 \
\\  ]^-1 \
\\ * [ 0; 13/50; 981/625; 254903/31250 ]

\\ vector(6,k,Ix(k-1))
Ix(k)  =  (3/656 - 7/328*I)*(-3/25 + 4/25*I)^k \
       +  (3/656 + 7/328*I)*(-3/25 - 4/25*I)^k \
       + 27/410*5^k \
       + -3/40;
\\ vector(6,k,Ix(k-1))

check_equal(vector(100,k,my(k=k-1); Ix(k)), \
            vector(100,k,my(k=k-1); Ix_by_rec(k)))

check_equal( vector(6,k,my(k=k-1); abs((-3/25 + 4/25*I)^k)), \
             [1, 1/5, 1/25, 1/125, 1/625, 1/3125] )
check_equal( vector(6,k,my(k=k-1); abs((-3/5 + 4/5*I)^k)), \
             [1, 1, 1, 1, 1, 1] )

\\-------------------

Ix(k) = + 54/820 * 5^k \
        +  3/328 * real((-3/25 + 4/25*I)^k) \
        + 14/328 * imag((-3/25 + 4/25*I)^k) \
        - 3/40;
check_equal(vector(100,k,my(k=k-1); Ix(k)), \
            vector(100,k,my(k=k-1); Ix_by_rec(k)))
check_equal(vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_sum(k,a))))
\\ vector(10,k,my(k=k-1); real((-3/5 + 4/5*I)^k))
\\ vector(10,k,my(k=k-1); Ix(k)*5^k)

Iy(k) = + 69/820 * 5^k \
        -  3/328 * real((-3/25 + 4/25*I)^k) \
        - 14/328 * imag((-3/25 + 4/25*I)^k) \
        - 3/40;
check_equal(vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_sum(k,a))))

{
Ixy(k) = - 7/82  * 5^k
         + 7/82  * real((-3/25 + 4/25*I)^k) \
         - 3/164 * imag((-3/25 + 4/25*I)^k) \
         ;  
}
check_equal(Ixy(0), 0)
check_equal(vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika(k,a))), \
            vector(20,a,my(a=2*Pi/a); vector(5,k,my(k=k-1); Ika_by_sum(k,a))))

\\-------------------
\\ Ixy -> -7/82
\\ Ix-Iy -> 54/820-69/820 = -3/164

\\plot(a=0,35, Ika(5,deg(a)))
plot(a=38.5,39.5, Ika_by_sum(5,deg(a)))

I_d(k) = 1/2*Ixy(k)  +  1/2*(Ix(k)-Iy(k))*I;
I_D(k)     = my(d=I_d(k)); if(d==0,0, abs(I_d(k)));
I_delta(k) = my(d=I_d(k)); if(d==0,0, arg(I_d(k)));
Ika_by_D(k,a) = 1/2*(Ix(k)+Iy(k)) + I_D(k)*sin(2*a + I_delta(k));

\\ I_d(k) = -7/82 - 3/164*I
\\ I_d(k) = -14/164 - 3/164*I
plot(a=0,90, Ika_by_D(5,deg(a)))
I_d(2)

\\-------------------
\\ (cos -sin)( Ix Ixy )( cos sin)
\\ (sin cos )( Ixy Iy )(-sin cos)
\\ =
\\ (cos -sin)(Ix*cos-Ixy*sin  Ix*sin+Ixy*cos)
\\ (sin cos )(Ixy*cos-Iy*sin  Ixy*sin-Iy*cos)
\\ =
\\ (Ix*cos^2 - Ixy*cos*sin - Ixy*cos*sin + Iy*sin^2,
\\  Ix*cos*sin + Ixy*cos^2 - Ixy*sin^2 + Iy*sin*cos;
\\ 
\\ Ix*cos*sin + Ixy*cos^2 - Ixy*sin^2 + Iy*sin*cos = 0
\\ (Ix+Iy)*cos*sin + Ixy*cos^2 - Ixy*sin^2 = 0
\\
\\ Rz(theta) = ( cos sin 0 )
\\             (-sin cos 0 )
\\             (  0   0  1 )
\\
\\
\\       Ix + Iy     Ix - Iy            Ixy + Iyx
\\ Ix' = -------  +  ------- * cos 2t + --------- * sin 2t
\\          2           2                   2
\\
\\ when Ixy=Iyx Mohr's circle relations
\\
\\          Ix - Iy  
\\ Ixy' = - ------- sin 2t  +  Ixy cos 2t
\\             2
\\
\\ (Ix-Iy)/2 sin 2t = Ixy cos 2t
\\ sin 2t / cos 2t = Ixy / (Ix-Iy)/2
\\
\\ 2t = arg (Ix-Iy)/2 + Ixy * I
\\
\\ ( Ix-1 Ixy )(a) = (0)
\\ ( Ixy Iy-1 )(b)   (0)
\\ 