#!/usr/bin/perl -w

# Copyright 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon;
use MyGraphs;


my $max_level = 3;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";

my $var = 'gen';

my %arrays;
foreach my $k (0 .. $max_level) {
  my $graph = MyR5Dragon::r5quad_area_tree_Graph($k);
  my @diameter = MyGraphs::Graph_tree_diameter_path($graph);
  my $Cv = $diameter[0];
  $graph->has_vertex($Cv) or die;

  my %degree1 = map {$graph->vertex_degree($_)==1
                       ? ($_ => 1) : ()} $graph->vertices;

  my $QAW = 0;
  my $QAwC = 0;
  my $QApairs = 0;

  my $QATW = 0;
  my $QAtwC = 0;
  my $QADeg1pairs = 0;

  $graph->for_shortest_paths
    (sub {
       my ($t, $u,$v, $n) = @_;
       my $path_length = $t->path_length($u,$v);
       if ($u lt $v) {  # one direction only
         $QAW += $path_length;
         $QApairs++;

         if ($degree1{$u} && $degree1{$v}) {
           $QATW += $path_length;
           $QADeg1pairs++;
         }
       }
       if ($u eq $Cv) {
         $QAwC += $path_length;
         if ($degree1{$v}) { $QAtwC += $path_length; }
       }
     });

  push @{$arrays{'QAW'}},   $QAW;
  push @{$arrays{'QAwC'}},  $QAwC;
  push @{$arrays{'QApairs'}}, $QApairs;

  push @{$arrays{'QATW'}},   $QATW;
  push @{$arrays{'QAtwC'}},  $QAtwC;
  push @{$arrays{'QADeg1pairs'}}, $QADeg1pairs;
}

foreach my $name ('QAW',   'QApairs',  'QAwC',
                  'QATW', 'QADeg1pairs', 'QAtwC',
                 ) {
  my $aref = $arrays{$name};
  print "% GP-DEFINE  ${var}_${name}_samples = [", join(',',@$aref),"];\n";
}

print "% end generated\n";
exit 0;
