#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;
use List::Util 'min';
use Math::PlanePath::R5DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon 'chamfer','J','xy_rotate_plus90';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 2;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::R5DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my $x_one = 3**($level/2);
### $x_hi
### $y_hi

sub n_is_join {
  my ($n) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_<=2*$n_hi} @n_list;
  my $below = grep {$_ < $n_hi} @n_list;
  my $above = grep {$_ > $n_hi} @n_list;
  return ($below == 1 && $above == 1);
}

print "  \\node (start) at (0,0) [my small dot];\n";
print "  \\node (end) at ($x_hi,$y_hi) [my small dot];\n";

{
  # lines
  $Printwrap = 2;
  printwrap("\\draw (start)");
  $Printwrap = 4;
  my %seen;
  my $count = 0;
  foreach my $n ($n_lo+1 .. 2*$n_hi) {
    my ($x,$y) = $path->n_to_xy($n);

    if (n_is_join($n)) {
      {
        # before $n
        my ($dx,$dy) = $path->n_to_dxdy($n-1);
        printwrap("--($x".chamfer(-$dx).",$y".chamfer(-$dy).")");
      }
      {
        # after $n
        my ($dx,$dy) = $path->n_to_dxdy($n);
        printwrap("--($x".chamfer($dx).",$y".chamfer($dy).")");
      }
    } else {
      printwrap("--($x,$y)");
    }
  }
  printwrap(";\n");
}

{
  # squares
  $Printwrap = 2;
  printwrap("\\path[fill,my join colour] foreach \\p in {");
  $Printwrap = 4;
  my %seen;
  my $total_x = 0;
  my $total_y = 0;
  my $count = 0;
  foreach my $n ($n_lo .. $n_hi-1) {
    next unless $path->_UNDOCUMENTED__n_segment_is_left_boundary($n, $level);
    next unless n_is_join($n);

    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    my $dx = $x2-$x1;
    my $dy = $y2-$y1;

    my ($dxL,$dyL) = xy_rotate_plus90($dx,$dy);
    my $xm = $x1 + $dxL;
    my $ym = $y1 + $dyL;
    my ($x3,$y3) = ($x1+$dxL, $y1+$dyL);
    my ($x4,$y4) = ($x2+$dxL, $y2+$dyL);

    my $mx = min($x1,$x2,$x3,$x4);
    my $my = min($y1,$y2,$y3,$y4);
    next if $seen{"$mx,$my"}++;

    printwrap("($mx,$my),");
    $count++;

    $total_x += $mx + .5;
    $total_y += $my + .5;
  }
  printwrap("}");
  printwrap(" { \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");

  my $J = J($level);
  $count == $J or die "oops level=$level count=$count J=$J";

  my $GJ_x = "$total_x/$count";
  my $GJ_y = "$total_y/$count";
  print "  \\node (GJ) at ($GJ_x,$GJ_y) [my dot];\n";
  print "  \\newcommand{\\MyCount}{$count}\n";
  print "% GP-Test  J($level) == $count   /* join squares */\n";
}

print "  % end generated\n";
exit 0;
