#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use Math::PlanePath::R5DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyR5Dragon '@dir4_to_dx','@dir4_to_dy','chamfer';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::R5DragonCurve->new;

$Printwrap = 2;
my $n_lo = 197;
my $n_hi = $n_lo + 5;

my $segments = MyLineSegments->new;
my ($x_lo,$y_lo) = $path->n_to_xy($n_lo);
print "  \\coordinate (start) at ($x_lo,$y_lo);\n";

$Printwrap = 2;
my $join = '';
my $start_from;
my $prev;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  $segments->add($x,$y, $x2,$y2);


  my $from = sprintf "%d%s,%d%s", $x,chamfer($dx), $y,chamfer($dy);
  my $to = sprintf "%d%s,%d%s", $x2,chamfer(-$dx), $y2,chamfer(-$dy);
  if ($prev) {
    printwrap(" \\draw[my segment join]($prev)--($from);");
  }

  printwrap(" \\draw[->,my segment]($from)--($to);");
  $prev = $to;
  $start_from //= $from;
}
printwrap_newline();
$Printwrap = 2;

# within $n_hi
sub path_is_traversed {
  my ($x1,$y1, $x2,$y2) = @_;
  my $n = $path->xyxy_to_n_either ($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_hi;
}

my @points;
$Printwrap = 2;
printwrap("\\draw[dashed]");
$Printwrap = 4;
foreach my $n ($n_lo .. $n_hi) {
  foreach my $dir4 (0 .. 3) {
    my ($x,$y) = $path->n_to_xy($n);
    my $dx = $dir4_to_dx[$dir4];
    my $dy = $dir4_to_dy[$dir4];
    my ($x2,$y2) = ($x+$dx, $y+$dy);

    next unless path_is_traversed($x,$y, $x2,$y2);
    next if $segments->is_traversed($x,$y, $x2,$y2);

    if ($x == $x_lo && $y == $y_lo) {
      my $from = sprintf "%d%+d*\\MyChamfer,%d%+d*\\MyChamfer", $x,$dx, $y,$dy;
      printwrap(" ($start_from)--($from)--($x2,$y2)");
    } else {
      printwrap(" ($x,$y) -- +($dx,$dy)");
    }
    $segments->add($x,$y, $x2,$y2);
    push @points, [$x2,$y2];
  }
}

foreach my $p (@points) {
  my ($x,$y) = @$p;
  foreach my $p2 (@points) {
    my ($x2,$y2) = @$p2;

    next unless path_is_traversed($x,$y, $x2,$y2);
    next if $segments->is_traversed($x,$y, $x2,$y2);
    printwrap(" ($x,$y)--($x2,$y2)");
    $segments->add($x,$y, $x2,$y2);
  }
}

foreach my $p (@points) {
  my ($x,$y) = @$p;
  foreach my $dir4 (0 .. 3) {
    my $dx = $dir4_to_dx[$dir4];
    my $dy = $dir4_to_dy[$dir4];
    my ($x2,$y2) = ($x+$dx, $y+$dy);

    next unless path_is_traversed($x,$y, $x2,$y2);
    next if $segments->is_traversed($x,$y, $x2,$y2);

    printwrap(" ($x,$y) -- +($dx*\\MyEnd,$dy*\\MyEnd)");
  }
}

printwrap(";\n");  # end of dashed

print "% GP-Test  Epred(",$n_lo-1,") == 0\n";
print "% GP-Test  Epred($n_lo) == 1\n";
print "% GP-Test  Epred($n_hi) == 1\n";
print "% GP-Test  Epred(",$n_hi+1,") == 0\n";
print "  % end generated\n";
exit 0;
