#!/usr/bin/perl -w

# Copyright 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min';
use Math::PlanePath::R5DragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyR5Dragon 'xy_rotate_plus90';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";

sub transform {
  my ($x,$y) = @_;
  return xy_rotate_plus90($x,$y);
}

my $path = Math::PlanePath::R5DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

print "  \\coordinate (start) at (0,0);\n";
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = transform($x,$y);
  print "  \\coordinate (end) at ($x,$y);\n";
}

{
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi-1) {
    $segments->add($path->n_to_xy($n), $path->n_to_xy($n+1));
  }
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap (transform => \&transform);
  printwrap(";");
  my $count = $segments->lines_joined_count_points;
  printwrap(" % $count points\n");
}

print "  % end generated\n";
exit 0;
