#!/usr/bin/perl -w

# Copyright 2013, 2014, 2015, 2016, 2017, 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.


# Usage: perl test.pl
#
# Run some self-tests on ../r5dragon.tex and on MyR5Dragon.pm.
#

use 5.004;
use strict;
use File::Slurp;
use IPC::Run;
use List::Util 'max','min';
use Math::Matrix;
use Math::PlanePath::R5DragonCurve;
use Math::NumSeq::Fibonacci;
use Math::Geometry::Planar;

use FindBin;
use lib::abs $FindBin::Bin;
use MyR5Dragon ':all';


my $source_dir = File::Spec->catdir ($FindBin::Bin, File::Spec->updir);
$source_dir = File::Spec->rel2abs($source_dir);
print "source dir $source_dir\n";

chdir $source_dir or die $!;

my $r5dragon_tex_filename = 'r5dragon.tex';
# $r5dragon_tex_filename = 'devel/more.tex';
my $r5dragon_pdf_filename = 'r5dragon.pdf';
my $r5dragon_tex_str = File::Slurp::slurp($r5dragon_tex_filename);
print count_lines($r5dragon_tex_str)," lines\n";

# my $r5dragon_tex_str_sans_comments = $r5dragon_tex_str;
# $r5dragon_tex_str_sans_comments =~ s/((^|[^\\])(\\\\)*)%[^\n]*/$1/mg;
# print count_lines($r5dragon_tex_str_sans_comments)," lines\n";


#------------------------------------------------------------------------------
# r5dragon.tex copyright years also at:
#    text part of r5dragon.tex
#    ../devel/r5dragon-defines.gp
#    ../devel/r5dragon-defines.header.gp

{
  # same copyright years as r5dragon.tex where defines are extracted from
  $r5dragon_tex_str =~ /^% (Copyright .*)/m or die;
  my $want_copyright = $1;
  ### $want_copyright

  {
    $r5dragon_tex_str =~ /^\\footnote.*?\n\Q$want_copyright\E\.$/m
      or die "r5dragon.tex: header comment copyright not found as footnote";
  }

  foreach my $filename ('devel/r5dragon-defines.gp',
                        'devel/r5dragon-defines.header.gp') {
    my $str = File::Slurp::slurp($filename);
    $str =~ /^\\\\ \Q$want_copyright\E$/m
      or die "$filename copyright different from r5dragon.tex";
  }
}


#------------------------------------------------------------------------------
# r5dragon.pdf Title and Author

{
  IPC::Run::run(['pdfinfo', '-isodates', $r5dragon_pdf_filename],
                '>', \my $pdfinfo_out) or die;
  $pdfinfo_out =~ s/^Jav.*\n//m;
  print $pdfinfo_out;
  {
    $pdfinfo_out =~ /^Title:\s*(.*)/m
      or die;
    my $title = $1;
    # print "title: $title\n";
    $title =~ /R5/
      or die;
  }
  {
    $pdfinfo_out =~ /^Author:\s*(.*)/m
      or die;
    my $author = $1;
    # print "author: $author\n";
    $author =~ /Kevin/
      or die;
  }
}

#------------------------------------------------------------------------------
sub pos_linenum {
  my ($str) = @_;
  $str = substr($str, 0, pos($_[0]));
  return 1 + scalar($str =~ tr/\n//);
}

sub count_lines {
  my ($str) = @_;
  my $count = ($str =~ tr/\n/\n/);
  if ($str =~ /[^\n]$/) { $count++; }
  return $count;
}

#------------------------------------------------------------------------------
exit 0;
