#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use List::Util 'sum';
use Math::BaseCnv 'cnv';
use Math::PlanePath::QuintetReplicate 125; # v.125 for numbering_type=>'rotate'

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyR5Dragon 'xy_rotate_plus60','xy_rotate_plus90';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
use Smart::Comments;

my $level = 2;
my $numbering_type = 'fixed';
if (@ARGV) { $numbering_type = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $numbering_type\n";

my $all_segments = MyLineSegments->new;
{
  my $path = Math::PlanePath::QuintetReplicate->new
    (numbering_type => $numbering_type);
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    $x *= 3;
    $x += 2;
    my ($dx,$dy) = (-1,1);
    foreach (1 .. 4) {
      my $x2 = $x+$dx;
      my $y2 = $y+$dy;
      $all_segments->add($x,$y, $x2,$y2);
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
      ($x,$y) = ($x2,$y2);
    }
  }
}
{
  my $segments = MyLineSegments->new;
  my $path = Math::PlanePath::GosperIslands->new;
  my $n_lo = 3**($level+0) - 2;
  my $n_hi = 3**($level+1) - 2 - 1;
  my ($dx,$dy) = (0,0);
  foreach my $a (0 .. 6) {
    foreach my $n ($n_lo .. $n_hi) {
      my $n_to = ($n==$n_hi ? $n_lo : $n+1);
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n_to);
      $segments->add($x1+2*$dx,$y1+2*$dy, $x2+2*$dx,$y2+2*$dy);
    }
    if ($a == 0) {
      ($dx,$dy) = (3,2);
    } else {
      ($dx,$dy) = xy_rotate_plus60($dx,$dy);
    }
  }

  print "  \\begin{scope}[xscale=1/2,yscale=1/sqrt(3)/2]\n";
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap (transform => \&xy_rotate_plus90);
  printwrap(";\n");

  $all_segments->remove_segments($segments);
  $Printwrap = 2;
  printwrap("\\draw[my inner] ");
  $Printwrap += 2;
  $all_segments->printwrap (transform => \&xy_rotate_plus90);
  printwrap(";\n");
  print "  \\end{scope}\n";
}

{
  my @samples;
  my $path = Math::PlanePath::QuintetReplicate->new
    (numbering_type => $numbering_type);
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  print "  \\begin{scope}[xscale=.5,yscale={sqrt(3)/2},font=\\scriptsize]\n";
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my $n7 = cnv($n,10,7);
    printwrap( " \\node at ($x,$y) {$n7};");
    push @samples, "$x/2+sqrt3i/2*$y";
  }
  printwrap_newline();
  print "  \\end{scope}\n";

  printwrap_indent("% GP-DEFINE  ");
  $Printwrap = 0;
  printwrap("IslandPoint_${numbering_type}_generated_samples = {[");
  $Printwrap = 2;
  foreach my $i (0 .. $#samples) {
    printwrap($samples[$i] . ($i==$#samples ? '' : ','));
  }
  $Printwrap = 0;
  printwrap("]};\n");
}

print "  % end generated\n";
exit 0;
